import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';
import { menu } from './menu';

const csvUrl = [
  'https://gist.githubusercontent.com/',
  'curran/',
  'a08a1080b88344b0c8a7/',
  'raw/0e7a9b0a5d22642a06d3d5b9bcbad9890c8ee534/',
  'iris.csv',
].join('');

const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

const width = window.innerWidth;
const height = window.innerHeight;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const menuContainer = select('body')
  .append('div')
  .attr('class', 'menu-container');

const xMenu = menuContainer
  .append('div')
  .attr('class', 'x-menu-container');

const yMenu = menuContainer
  .append('div')
  .attr('class', 'y-menu-container');

const margin = {
  top: 20,
  right: 20,
  bottom: 40,
  left: 50,
};

const main = async () => {
  const data = await csv(csvUrl, parseRow);
  


  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(data)
    .xValue((d) => d.petal_width)
    .yValue((d) => d.petal_width)
    .margin(margin)
    .size(5);

  svg.call(plot);

  // MEnu
  const options = [
    {
      value: 'petal_width',
      text: 'Petal Width',
      type: 'quantitative',
    },
    {
      value: 'sepal_width',
      text: 'Sepal Width',
      type: 'quantitative',
    },
    {
      value: 'petal_length',
      text: 'Petal Length',
      type: 'quantitative',
    },
    {
      value: 'sepal_length',
      text: 'Sepal Length',
      type: 'quantitative',
    },
    {
      value: 'species',
      text: 'Species',
      type: 'categorical',
    },
  ];

  const columnToType = new Map(
    options.map(({ value, type }) => [
      value,
      type,
    ])
  );

  const getType = (column) =>
    columnToType.get(column);

  xMenu.call(
    menu()
      .id('x-menu')
      .labelText('X: ')
      .options(options)
      .on('change', (columnName) => {
        svg.call(
          plot
            .xValue((d) => d[columnName])
            .xType(getType(columnName))
        );
      })
  );

  yMenu.call(
    menu()
      .id('y-menu')
      .labelText('Y: ')
      .options(options)
      .on('change', (columnName) => {
        svg.call(
          plot
            .yValue((d) => d[columnName])
            .yType(getType(columnName))
        );
      })
  );
};
main();
