import {
  scaleLinear,
  scalePoint,
  min,
  max,
  extent,
  axisLeft,
  axisBottom,
  transition,
} from 'd3';

export const scatterPlot = () => {
  let width = 960;
  let height = 500;
  let data = [];
  let xValue = [];
  let yValue = [];
  let xType = '';
  let yType = '';
  let margin = {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0,
  };
  let size = 10;

  const my = (selection) => {
    // Transition
    const t = transition().duration(1000);

    // X-axis
    const xScale = (xType === 'categorical'
      ? scalePoint().domain(data.map(xValue)).padding(0.2)
      : scaleLinear().domain(extent(data, xValue))
    ).range([margin.left, width - margin.right]);

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .transition(t)
      .call(axisBottom(xScale));

    // Y-axis
    const yScale = (yType === 'categorical'
      ? scalePoint().domain(data.map(yValue)).padding(0.2)
      : scaleLinear().domain(extent(data, yValue))
    ).range([height - margin.bottom, margin.top]);

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr(
        'transform',
        `translate(${margin.left}, 0)`
      )
      .transition(t)
      .call(axisLeft(yScale));

    // Position on Canvas
    const marks = data.map((d) => ({
      x: xScale(xValue(d)),
      y: yScale(yValue(d)),
    }));

    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join(
        (enter) =>
          enter
            .append('circle')
            .attr('cx', (d) => d.x)
            .attr('cy', (d) => d.y)
            .attr('r', 0)
            .call((enter) =>
              enter.transition(t).attr('r', size)
            ),
        (update) =>
          update.call((update) =>
            update
              .transition(t)
              .delay((d, i) => i * 10)
              .attr('cx', (d) => d.x)
              .attr('cy', (d) => d.y)
          ),
        (exit) => exit.remove()
      );
  };

  // Getters
  my.width = function (_) {
    return arguments.length
      ? ((width = +_), my)
      : width;
  };
  my.height = function (_) {
    return arguments.length
      ? ((height = +_), my)
      : height;
  };
  my.data = function (_) {
    return arguments.length
      ? ((data = _), my)
      : data;
  };
  my.xValue = function (_) {
    return arguments.length
      ? ((xValue = _), my)
      : xValue;
  };
  my.yValue = function (_) {
    return arguments.length
      ? ((yValue = _), my)
      : yValue;
  };
  my.xType = function (_) {
    return arguments.length
      ? ((xType = _), my)
      : xType;
  };
  my.yType = function (_) {
    return arguments.length
      ? ((yType = _), my)
      : yType;
  };
  my.margin = function (_) {
    return arguments.length
      ? ((margin = _), my)
      : margin;
  };
  my.size = function (_) {
    return arguments.length
      ? ((size = +_), my)
      : size;
  };

  return my;
};
